﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Windows.Forms;

namespace WindowsFormsApplication1
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();
		}

		private void Form1_Shown(object sender, EventArgs e)
		{
			// report object
			FastReport.Report report = new FastReport.Report
			{
				Preview = previewControl1
			};

			XmlDocument doc = new XmlDocument();
			doc.Load(@"..\..\..\nwind.xml");

			List<MatrixDemo> matrixDemo = new List<MatrixDemo>();
			XmlNodeList root = doc.SelectNodes("//MatrixDemo");
			foreach (XmlNode item in root)
			{
				XmlSerializer serializer = new XmlSerializer(typeof(MatrixDemo), new XmlRootAttribute("MatrixDemo"));
				using (StringReader sr = new StringReader(item.OuterXml))
				{
					matrixDemo.Add(serializer.Deserialize(sr) as MatrixDemo);
				}
			}

			report.RegisterData(matrixDemo, "MatrixDemo", maxNestingLevel: 1);

			foreach (FastReport.Data.DataSourceBase item in report.Dictionary.DataSources)
			{
				item.Enabled = true;
			}

			// add report page
			FastReport.ReportPage page = new FastReport.ReportPage
			{
				LeftMargin = 10,
				RightMargin = 10,
				TopMargin = 10,
				BottomMargin = 10,
				UnlimitedWidth = true
			};
			report.Pages.Add(page);

			// create data band
			FastReport.DataBand dataBand = new FastReport.DataBand
			{
				Height = FastReport.Utils.Units.Centimeters * 0.5f
			};
			page.Bands.Add(dataBand);

			// create child band
			FastReport.ChildBand childBand = new FastReport.ChildBand()
			{
				Height = FastReport.Utils.Units.Centimeters * 1.75f,
				Parent = dataBand
			};

			// table object
			FastReport.Table.TableObject tableObject = new FastReport.Table.TableObject()
			{
				Name = "Table1",
				Width = FastReport.Utils.Units.Centimeters * 8.75f,
				Height = FastReport.Utils.Units.Centimeters * 0.5f,
				Parent = dataBand,
				ManualBuildEvent = "Table1_ManualBuild"
			};

			FastReport.Table.TableRow tableRow = new FastReport.Table.TableRow
			{
				Parent = tableObject
			};
			FastReport.Table.TableColumn tableColumn1 = new FastReport.Table.TableColumn
			{
				Width = FastReport.Utils.Units.Centimeters * 1.75f,
				Parent = tableObject
			};
			FastReport.Table.TableColumn tableColumn2 = new FastReport.Table.TableColumn
			{
				Width = FastReport.Utils.Units.Centimeters * 1.75f,
				Parent = tableObject
			};
			FastReport.Table.TableColumn tableColumn3 = new FastReport.Table.TableColumn
			{
				Width = FastReport.Utils.Units.Centimeters * 1.75f,
				Parent = tableObject
			};
			FastReport.Table.TableColumn tableColumn4 = new FastReport.Table.TableColumn
			{
				Width = FastReport.Utils.Units.Centimeters * 1.75f,
				Parent = tableObject
			};
			FastReport.Table.TableColumn tableColumn5 = new FastReport.Table.TableColumn
			{
				Width = FastReport.Utils.Units.Centimeters * 1.75f,
				Parent = tableObject
			};
			FastReport.Table.TableCell tableCell1 = new FastReport.Table.TableCell
			{
				Text = "[MatrixDemo.Name]",
				Parent = tableRow
			};
			FastReport.Table.TableCell tableCell2 = new FastReport.Table.TableCell
			{
				Text = "[MatrixDemo.Year]",
				Parent = tableRow
			};
			FastReport.Table.TableCell tableCell3 = new FastReport.Table.TableCell
			{
				Text = "[MatrixDemo.Month]",
				Parent = tableRow
			};
			FastReport.Table.TableCell tableCell4 = new FastReport.Table.TableCell
			{
				Text = "[MatrixDemo.ItemsSold]",
				Parent = tableRow
			};
			FastReport.Table.TableCell tableCell5 = new FastReport.Table.TableCell
			{
				Text = "[MatrixDemo.Revenue]",
				Parent = tableRow
			};

			// matrix object
			FastReport.Matrix.MatrixObject matrix = new FastReport.Matrix.MatrixObject
			{
				Bounds = new RectangleF(FastReport.Utils.Units.Centimeters * 0.1f, FastReport.Utils.Units.Centimeters * 0.1f, FastReport.Utils.Units.Centimeters * 8f, FastReport.Utils.Units.Centimeters * 2f),
				CellsSideBySide = true,
				DataSource = report.GetDataSource("MatrixDemo"),
				Style = "blue",
				Parent = childBand
			};

			// create one column descriptor
			FastReport.Matrix.MatrixHeaderDescriptor column = new FastReport.Matrix.MatrixHeaderDescriptor("[MatrixDemo.Year]");
			matrix.Data.Columns.Add(column);

			// create one row descriptor
			FastReport.Matrix.MatrixHeaderDescriptor row = new FastReport.Matrix.MatrixHeaderDescriptor("[MatrixDemo.Name]");
			matrix.Data.Rows.Add(row);

			// create data cell
			FastReport.Matrix.MatrixCellDescriptor cell1 = new FastReport.Matrix.MatrixCellDescriptor("[MatrixDemo.ItemsSold]", FastReport.Matrix.MatrixAggregateFunction.Max);
			matrix.Data.Cells.Add(cell1);

			// build matrix template
			matrix.BuildTemplate();

			// change header text
			FastReport.Table.TableCell tableCell = (FastReport.Table.TableCell)matrix.Rows[0][0];
			tableCell.Text = "Employee";
			tableCell = (FastReport.Table.TableCell)matrix.Rows[0][2];
			tableCell.Text = "Sum";

			// autosize column
			matrix.Columns[0].AutoSize = true;
			matrix.Columns[1].AutoSize = true;
			matrix.Columns[2].AutoSize = true;

			// delete row
			matrix.Rows[2].Height = 0;

			// assign an event on table cell
			tableCell = (FastReport.Table.TableCell)matrix.Rows[1][1];
			tableCell.BeforePrintEvent = "Cell5_BeforePrint";
			tableCell = (FastReport.Table.TableCell)matrix.Rows[1][2];
			tableCell.BeforePrintEvent = "Cell6_BeforePrint";

			report.ScriptText = @"using System;
using FastReport;
using FastReport.Data;
using FastReport.Table;
namespace FastReport
{
	public class ReportScript
	{
		private int sumQty;

		private void Cell5_BeforePrint(object sender, EventArgs e)
		{
			if (Cell5.Value != null)
			{
				sumQty = sumQty + (int)Cell5.Value;
			}
		}
		private void Cell6_BeforePrint(object sender, EventArgs e)
		{
			Cell6.Text = sumQty.ToString();
			sumQty = 0;
		}
		private void Table1_ManualBuild(object sender, EventArgs e)
		{
			DataSourceBase data = Report.GetDataSource(""MatrixDemo"");
			data.Init();
			data.First();
			while (data.HasMoreRows)
			{
				Table1.PrintRow(0);
				Table1.PrintColumns();
				data.Next();
			}
		}
	}
}";
			report.Save("matrix.frx");
			report.Show();
		}

		public class MatrixDemo
		{
			public string Name { get; set; }
			public int Year { get; set; }
			public int Month { get; set; }
			public int ItemsSold { get; set; }
			public decimal Revenue { get; set; }
		}
	}
}